/* This is an example workflow that shows how HDF would use the IOD API
 Based on the H5File shown on previous page taken originally from
 Ruth's figure and then augmented with the big red numbers which indicate
 the array offset where their object ids get stored.
 It's pseudo-code but pretty close to how this code would work except:
     1: IGNORES ERROR CODES
     2: DOES EVERYTHING SYNCHRONOUSLY
     3: PERL/PYTHON STYLE ANONYMOUS ARRAYS for clarity
*/

#include "iod_api.h"
#include "stdlib.h"

// some definitions to make the code more readable
#define IOD_SYNCHRONOUS NULL
#define NO_TRANSACTION  0
#define NO_STRUCT NULL
#define NONAME NULL

int
main(int argc, char **argv) {
    // **** STEP 1: Create an H5File at /scratch/foo **********************
    iod_hint_list_t hints;        // populate this.  (not shown)
    iod_handle_t coh;
    //iod_mode_t mode;
    iod_event_t event;
    int mpi_size, mpi_rank;
    MPI_Comm comm  = MPI_COMM_WORLD;

    MPI_Init(&argc, &argv);
    MPI_Comm_size(comm, &mpi_size);
    MPI_Comm_rank(comm, &mpi_rank);

    iod_initialize(comm, &hints, mpi_size, mpi_size, &event);

    iod_container_open("/scratch/foo", &hints, IOD_CONT_CREATE|IOD_CONT_WO,
		       &coh, IOD_SYNCHRONOUS);


    // **** STEP 2: Create all the objects inside the container ***********
    // This could be done in parallel across CN's if desired.
    // Object creation for named objects is the same as container creation:
    // Either all can do asynchronously, or one can do and broadcast.
    iod_obj_id_t kvs[6];            // create six key-value objects
    iod_obj_id_t arrays[9];         // create eight array objects
    iod_obj_id_t blobs[1];          // create one blob object
    iod_ret_t rets[6+9+1];          // one for each object
    iod_array_struct_t structs[9]; // for arrays; populating them not shown
    //iod_size_t blob_size=128;
    // this example uses same hints for entire call and each object
    iod_obj_create_t obj_create[6+9+1] = {
        {IOD_OBJ_KV,   NONAME,       NULL,&hints, &kvs[0],   &rets[0]},
        {IOD_OBJ_KV,   NONAME,       NULL,&hints, &kvs[1],   &rets[1]},
        {IOD_OBJ_KV,   NONAME,       NULL,&hints, &kvs[2],   &rets[2]},
        {IOD_OBJ_KV,   NONAME,       NULL,&hints, &kvs[3],   &rets[3]},
        {IOD_OBJ_KV,   NONAME,       NULL,&hints, &kvs[4],   &rets[4]},
        {IOD_OBJ_KV,   NONAME,       NULL,&hints, &kvs[5],   &rets[5]},
        {IOD_OBJ_ARRAY,NONAME,&structs[0],&hints,&arrays[0],&rets[6],},
        {IOD_OBJ_ARRAY,NONAME,&structs[1],&hints,&arrays[1],&rets[7],},
        {IOD_OBJ_ARRAY,NONAME,&structs[2],&hints,&arrays[2],&rets[8],},
        {IOD_OBJ_ARRAY,NONAME,&structs[3],&hints,&arrays[3],&rets[9],},
        {IOD_OBJ_ARRAY,NONAME,&structs[4],&hints,&arrays[4],&rets[10]},
        {IOD_OBJ_ARRAY,NONAME,&structs[5],&hints,&arrays[5],&rets[11]},
        {IOD_OBJ_ARRAY,NONAME,&structs[6],&hints,&arrays[6],&rets[12]},
        {IOD_OBJ_ARRAY,NONAME,&structs[7],&hints,&arrays[7],&rets[13]},
        {IOD_OBJ_ARRAY,NONAME,&structs[8],&hints,&arrays[8],&rets[14]},
        {IOD_OBJ_BLOB, NONAME,       NULL,&hints, &blobs[0],&rets[15]},
    };

    iod_obj_create_list(
      coh,
      NO_TRANSACTION,
      6+9+1,
      obj_create,
      IOD_SYNCHRONOUS
    );

    iod_handle_t arrays_h[9];
    iod_handle_t blobs_h[1];
    iod_handle_t kvs_h[6];
    iod_obj_open_t obj_open[9+1+6] = {
	{arrays[0], &hints, &arrays_h[0], &rets[0]},
	{arrays[1], &hints, &arrays_h[1], &rets[1]},
	{arrays[2], &hints, &arrays_h[2], &rets[2]},
	{arrays[3], &hints, &arrays_h[3], &rets[3]},
	{arrays[4], &hints, &arrays_h[4], &rets[4]},
	{arrays[5], &hints, &arrays_h[5], &rets[5]},
	{arrays[6], &hints, &arrays_h[6], &rets[6]},
	{arrays[7], &hints, &arrays_h[7], &rets[7]},
	{arrays[8], &hints, &arrays_h[8], &rets[8]},
	{blobs[0],  &hints,  &blobs_h[0], &rets[9]},
	{kvs[0],    &hints,    &kvs_h[0], &rets[10]},
	{kvs[1],    &hints,    &kvs_h[1], &rets[11]},
	{kvs[2],    &hints,    &kvs_h[2], &rets[12]},
	{kvs[3],    &hints,    &kvs_h[3], &rets[13]},
	{kvs[4],    &hints,    &kvs_h[4], &rets[14]},
	{kvs[5],    &hints,    &kvs_h[5], &rets[15]}
    };
    iod_obj_open_write_list(
        coh,
	9+1+6,
        obj_open,
	NULL
    );
    // **** STEP 3: Create the links using key-values **********************
    // Don't use the extra metadata bit right now
    size_t oid_sz = sizeof(iod_obj_id_t); // reuse this variable a bunch
    iod_kv_t kv_pairs0[4] = {
        { "ID",             (void*)"XX",    2     },
        { "Verified",       (void*)"Y",     1     },
        { "visualizations", (void*)&kvs[1], oid_sz},
        { "mesh",           (void*)&kvs[2], oid_sz}
    };
    int ret_kvset[6];
    iod_kv_params_t kv_parames0[4] = {
	{&kv_pairs0[0], NULL, &ret_kvset[0]},
	{&kv_pairs0[1], NULL, &ret_kvset[1]},
	{&kv_pairs0[2], NULL, &ret_kvset[2]},
	{&kv_pairs0[3], NULL, &ret_kvset[3]}
    };
    iod_kv_set_list (
        kvs_h[0],
        NO_TRANSACTION,
	&hints,
        4,
	kv_parames0,
        IOD_SYNCHRONOUS
    );
    iod_kv_t kv_pairs1[3] = {
        { "view1",  (void*)&arrays[0], oid_sz},
        { "view2",  (void*)&arrays[2], oid_sz},
        { "labels", (void*)&kvs[3],    oid_sz},
    };
    iod_kv_params_t kv_parames1[3] = {
	{&kv_pairs1[0], NULL, &ret_kvset[0]},
	{&kv_pairs1[1], NULL, &ret_kvset[1]},
	{&kv_pairs1[2], NULL, &ret_kvset[2]}
    };
    iod_kv_set_list(
        kvs_h[1],
        NO_TRANSACTION,
	&hints,
        3,
	kv_parames1,
        IOD_SYNCHRONOUS
    );
    iod_kv_t kv_pairs2[6] = {
        { "axes", (void*)&kvs[3],  oid_sz},
        { "step1", (void*)&kvs[4], oid_sz},
        { "step2", (void*)&kvs[5], oid_sz},
        { "last_step", (void*)&kvs[5], oid_sz},
        { "initial_values", (void*)&arrays[1],oid_sz},
        { "nodetype", (void*)&blobs[0], oid_sz},
    };
    iod_kv_params_t kv_parames2[6] = {
	{&kv_pairs2[0], NULL, &ret_kvset[0]},
	{&kv_pairs2[1], NULL, &ret_kvset[1]},
	{&kv_pairs2[2], NULL, &ret_kvset[2]},
	{&kv_pairs2[3], NULL, &ret_kvset[3]},
	{&kv_pairs2[4], NULL, &ret_kvset[4]},
	{&kv_pairs2[5], NULL, &ret_kvset[5]}
    };
    iod_kv_set_list(
        kvs_h[2],
        NO_TRANSACTION,
	&hints,
        6,
	kv_parames2,
        IOD_SYNCHRONOUS
    );
    iod_kv_t kv_pairs3[2] = {
        { "x", (void*)&arrays[3], oid_sz},
        { "y", (void*)&arrays[4], oid_sz},
    };
    iod_kv_params_t kv_parames3[2] = {
	{&kv_pairs3[0], NULL, &ret_kvset[0]},
	{&kv_pairs3[1], NULL, &ret_kvset[1]}
    };
    iod_kv_set_list(
        kvs_h[3],
        NO_TRANSACTION,
	&hints,
        2,
        kv_parames3,
        IOD_SYNCHRONOUS
    );
    iod_kv_t kv_pairs4[3] = {
        { "stats",    (void*)&arrays[5], oid_sz},
        { "values",   (void*)&arrays[6], oid_sz},
        { "previous", (void*)&kvs[2],    oid_sz},
    };
    iod_kv_params_t kv_parames4[3] = {
	{&kv_pairs4[0], NULL, &ret_kvset[0]},
	{&kv_pairs4[1], NULL, &ret_kvset[1]},
	{&kv_pairs4[2], NULL, &ret_kvset[2]}
    };
    iod_kv_set_list(
        kvs_h[4],
        NO_TRANSACTION,
	&hints,
        2,
	kv_parames4,
        IOD_SYNCHRONOUS
    );
    iod_kv_t kv_pairs5[3] = {
        { "stats",    (void*)&arrays[7], oid_sz},
        { "values",   (void*)&arrays[8], oid_sz},
        { "previous", (void*)&kvs[4],    oid_sz},
    };
    iod_kv_params_t kv_parames5[3] = {
	{&kv_pairs5[0], NULL, &ret_kvset[0]},
	{&kv_pairs5[1], NULL, &ret_kvset[1]},
	{&kv_pairs5[2], NULL, &ret_kvset[2]}
    };
    iod_kv_set_list(
        kvs_h[5],
        NO_TRANSACTION,
	&hints,
        2,
	kv_parames5,
        IOD_SYNCHRONOUS
    );


    iod_finalize(&hints, &event);
    MPI_Finalize();
    return 0;
}
