/*
 *
 */

#include <iod_internal.h>

/**
 * create an Event Queue
 *
 * \param eq [OUT]	returned EQ handle
 *
 * \return		zero on success, negative value if error
 */
int iod_eq_create(iod_handle_t *eqh)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Destroy an Event Queue.
 *
 * \param eqh [IN]	EQ to destroy
 * \param ev [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_eq_destroy(iod_handle_t eqh, iod_event_t *ev)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Retrieve completion events from an EQ
 *
 * \param eqh [IN]	EQ handle
 * \param wait_if [IN]	wait only if there's inflight event
 * \param timeout [IN]	how long is caller going to wait (micro-second)
 *			if \a timeout > 0,
 *			it can also be IOD_EQ_NOWAIT, IOD_EQ_WAIT.
 * \param n_events [IN]	size of \a events array, returned number of events
 *			should always be less than or equal to \a num
 * \param events [OUT]	pointer to returned events array
 *
 * \return		>= 0	returned number of events
 *			< 0	negative value if error
 */
iod_ret_t iod_eq_poll(iod_handle_t eqh, int wait_if, uint64_t timeout,
		      int n_events, iod_event_t **events)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}


/**
 * Try to abort operations associated with this event.
 *
 * \param ev [IN]	event to abort
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_event_abort(iod_event_t *ev)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Query how many outstanding events in EQ, if \a events is not NULL,
 * these events will be stored into it.
 * Events returned by query are still owned by IOD, it's not allowed to
 * finalize or free events returned by this function, but it's allowed
 * to call iod_event_abort() to abort inflight operation.
 * Also, status of returned event could be still in changing, for example,
 * returned "inflight" event can be turned to "completed" before acessing.
 * It's user's responsibility to guarantee that returned events would be
 * freed by polling process.
 *
 * \param eqh [IN]	EQ handle
 * \param query [IN]	query mode
 * \param n_events [IN]	size of \a events array
 * \param events [OUT]	pointer to returned events array
 * \return		>= 0	returned number of events
 *			 < 0	negative value if error
 */
int iod_eq_query(iod_handle_t eqh, iod_ev_query_t query, unsigned int n_events,
		 iod_event_t **events)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Initialize a new event for \a eq
 *
 * \param ev [IN]	event to initialize
 * \param eqh [IN]	where the event to be queued on
 *
 * \return		zero on success, negative value if error
 */
int iod_event_init(iod_event_t *ev, iod_handle_t eqh)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Finalize an event. If event has been passed into any IOD API, it can only
 * be finalized when it's been polled out from EQ, even it's aborted by
 * calling iod_event_abort().
 *
 * \param ev [IN]	event to finialize
 */
void iod_event_fini(iod_event_t *ev)
{
	IOD_ENTER;


	IOD_EXIT;
}
