/*
 *
 */

#include <iod_internal.h>

/**
 * Instantiate IOD service.
 *
 * It should be called collectively by every application process at initializing
 * phase.
 *
 * The \a total_cnranks and \a cnranks are for IOD optimization. For example
 * if one TID's num_ranks == total_cnranks it means that all CN ranks
 * participate it so IOD can use collective instead lots of P2P messages to
 * synchronize transaction status; the cnranks can help IOD to know how many
 * CN ranks are connected to it.
 * User can pass in zero if it does not know it, but \a total_cnranks and
 * \a cnranks should be provided with meaningful value or set as zero
 * simultaneously. If they are provided, for application with dynamic processes,
 * user should re-call this routine to notify IOD the new value after dynamic
 * processes are created.
 *
 * \param comm [IN]		Global MPI communicator among all IODs
 * \param hints[IN]		pointer to hints and can be NULL when no hint
 * \param total_cnranks[IN]	total number of CN ranks in application
 * \param cnranks[IN]		number of CN ranks that connect to this IOD
 * \param event [IN]		pointer to completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_initialize(iod_comm_t comm, iod_hint_list_t *hints,
			 unsigned int total_cnranks, unsigned int cnranks,
			 iod_event_t *event)
{
    iod_get_conf();
	IOD_ENTER;
	mlog(IOD_DAPI, "Params - total_cnranks: %d, cnranks: %d, "
	     "hints:%p, event: %p", total_cnranks, cnranks, hints, event);


	IOD_EXIT;
	return 0;
}

/**
 * Finalize IOD for resource freeing.
 *
 * It should be called collectively by every application process when user
 * program exiting.
 *
 * \param hints[IN]		pointer to hints and can be NULL when no hint
 * \param event [IN]		pointer to completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_finalize(iod_hint_list_t *hints, iod_event_t *event)
{
	IOD_ENTER;
	mlog(IOD_DAPI, "Params - hints:%p, event: %p", hints, event);


	IOD_EXIT;
	iod_release_conf(iod_get_conf());
	return 0;
}
