/*
 *
 */

#include <iod_internal.h>

/**
 * Write to one IOD array object.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param hints[IN]	pointer to hints and can be NULL when no hint
 * \param mem_desc [IN]	pointer to memory buffers descriptor
 * \param io_desc[IN]	pointer to io descripter
 * \param cs[IN]	passed in checksum pointer for the write
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_array_write(iod_handle_t oh, iod_trans_id_t tid,
			  iod_hint_list_t *hints, iod_mem_desc_t *mem_desc,
			  iod_array_iodesc_t *io_desc, iod_checksum_t *cs,
			  iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Write to a set of IOD array objects within one transaction.
 * It is a wrapper inside which looply calls iod_obj_write_struct.
 *
 * \param coh [IN]		container handle
 * \param tid [IN]		transaction ID
 * \param num [IN]		number of objects for writing
 * \param array_write [IN/OUT]	pointer list of packed parameters
 * \param event [IN]		pointer to completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_array_write_list(iod_handle_t coh, iod_trans_id_t tid,
			       iod_size_t num, iod_array_io_t *array_write,
			       iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Read from one IOD array object.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param hints[IN]	pointer to hints and can be NULL when no hint
 * \param mem_desc [IN]	pointer to memory buffers descriptor
 * \param io_desc[IN]	pointer to I/O descriptor
 * \param cs[IN/OUT]	returned checksum for the read
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_array_read(iod_handle_t oh, iod_trans_id_t tid,
			 iod_hint_list_t *hints, iod_mem_desc_t *mem_desc,
			 iod_array_iodesc_t *io_desc, iod_checksum_t *cs,
			 iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Read from a set of IOD array objects within one transaction.
 * It is a wrapper inside which looply calls iod_obj_read_struct.
 *
 * \param coh [IN]		container handle
 * \param tid [IN]		transaction ID
 * \param num [IN]		how many objects to read
 * \param array_read [IN/OUT]	pointer list of packed parameters
 * \param event [IN]		pointer to completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_array_read_list(iod_handle_t coh, iod_trans_id_t tid,
			      iod_size_t num, iod_array_io_t *array_read,
			      iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Get current structure(dimensionality and cellsize) of an IOD ARRAY object.
 *
 * Any rank can call it, commonly single rank calls it to query.
 *
 * \param oh [IN]		object handle
 * \param tid [IN]		transaction ID
 * \param obj_struct [IN/OUT]	pointer of object structure
 * \param event [IN]		pointer of completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t
iod_array_get_struct(iod_handle_t oh, iod_trans_id_t tid,
		     iod_array_struct_t *array_struct, iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Get current structure(dimensionality and cellsize) of a set of IOD objects
 * within one transaction.
 *
 * \param coh [IN]		container handle
 * \param tid [IN]		transaction ID
 * \param num [IN]		how many objects
 * \param get_struct [IN/OUT]	pointer list of packed parameters
 * \param event [IN]		pointer to completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_array_get_struct_list(iod_handle_t coh, iod_trans_id_t tid,
				    iod_size_t num,
				    iod_array_get_struct_t *get_struct,
				    iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Extend one IOD ARRAY object's dimensionality.
 * One data array object can be extended along the first dimension.
 * It will fail when the object handle has no writing permission.
 * It will fail for fixed dimensional array, or \a firstdim_len >
 * firstdim_max.
 *
 * Only single rank should call it to extend array object. Caller should aware
 * the possible race here: when one rank extends array, previous gotten array
 * structure (by calling iod_array_get_struct) by other ranks will be invalid.
 * TODO: needs to support shink?
 *
 * \param oh [IN]		object handle
 * \param tid [IN]		transaction ID
 * \param firstdim_len[IN]	the first dimension length
 * \param event [IN]		completion event pointer list
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_array_extend(iod_handle_t oh, iod_trans_id_t tid,
			   iod_size_t firstdim_len, iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Extend the dimensionality of a set of IOD ARRAY objects.
 * All the objects should in one container. It will fail when the object handle
 * has no writing permission.
 *
 * \param coh [IN]		container handle
 * \param tid [IN]		transaction ID
 * \param num [IN]		how many objects
 * \param obj_extend [IN/OUT]	pointer list of packed parameters
 * \param event [IN]		pointer to completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_array_extend_list(iod_handle_t coh, iod_trans_id_t tid,
				iod_size_t num, iod_obj_extend_t *obj_extend,
				iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}
