/*
 *
 */

#include <iod_internal.h>

/**
 * Set one Key-Value pair to one KV object.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param hints[IN]	pointer to hints and can be NULL when no hint
 * \param kv [IN]	KV pair pointer
 * \param cs[IN]	passed in checksum pointer for the KV pair
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_kv_set(iod_handle_t oh, iod_trans_id_t tid,
		     iod_hint_list_t *hints, iod_kv_t *kv, iod_checksum_t *cs,
		     iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Set a list of Key-Value pairs to one KV object.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param hints[IN]	pointer to hints and can be NULL when no hint
 * \param num [IN]	number of KV pairs
 * \param kvs [IN/OUT]	pointer to KV parameters packet list
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_kv_set_list(iod_handle_t oh, iod_trans_id_t tid,
			  iod_hint_list_t *hints, iod_size_t num,
			  iod_kv_params_t *kvs, iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Get the number of Key-Value pairs of one KV object.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param num [IN/OUT]	pointer of number of KV pairs
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_kv_get_num(iod_handle_t oh, iod_trans_id_t tid, iod_size_t *num,
			 iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Get a list of KV pairs from one KV object.
 * Caller needs to provide and free the buffer for KV-pairs.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param hints[IN]	pointer to hints and can be NULL when no hint
 * \param offset [IN]	offset of kv pair inside the KV object(begin from 0),
 *			ordered by ascending alphabetical order of key (from
 *			'a' to 'z').
 * \param num [IN]	number of KV pairs
 * \param kvs [IN/OUT]	pointer to KV parameters packet list
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_kv_get_list(iod_handle_t oh, iod_trans_id_t tid,
			  iod_hint_list_t *hints, iod_off_t offset,
			  iod_size_t num, iod_kv_params_t *kvs,
			  iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Retrieve a list of keys in the KV object.
 * Caller needs to provide and free the buffer for key list, every key is a
 * buffer of IOD_KV_KEY_MAXLEN length. Needs not malloc buffer for value
 * inside iod_kv_params_t, it will be ignored by this call.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param hints[IN]	pointer to hints and can be NULL when no hint
 * \param offset [IN]	offset of key inside the KV object(begin from 0),
 *			ordered by ascending alphabetical order (from 'a'
 *			to 'z').
 * \param num [IN]	number of KV pairs
 * \param kvs [IN/OUT]	pointer to KV parameters packet list
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_kv_list_key(iod_handle_t oh, iod_trans_id_t tid,
			  iod_hint_list_t *hints, iod_off_t offset,
			  iod_size_t num, iod_kv_params_t *kvs,
			  iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Get the value belong to the key from one KV object.
 * Caller needs to provide and free the buffer for value, if user passed in
 * buffer length \a len is not enough to hold the value then when this routine
 * returns the \a len is set as the actual length of value.
 *
 * \param oh [IN]		object handle
 * \param tid [IN]		transaction ID
 * \param key [IN]		passed in key
 * \param value [IN/OUT]	returned value
 * \param len [IN/OUT]		pointer of length of passed in buffer of value
 * \param event [IN]		pointer to completion event
 *
 * \return			zero on success, negative value if error
 */
iod_ret_t iod_kv_get_value(iod_handle_t oh, iod_trans_id_t tid, const char *key,
			   char *value, iod_size_t *len, iod_checksum_t *cs,
			   iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}

/**
 * Unlink a list of keys in the container.
 * Caller needs to provide and free the buffer for key list, every key is a
 * NULL termed string(within length of IOD_KV_KEY_MAXLEN). Needs not malloc
 * buffer for value inside iod_kv_params_t, it will be ignored by this call.
 *
 * \param oh [IN]	object handle
 * \param tid [IN]	transaction ID
 * \param hints[IN]	pointer to hints and can be NULL when no hint
 * \param num [IN]	number of KV pairs
 * \param kvs [IN/OUT]	pointer to KV parameters packet list
 * \param event [IN]	pointer to completion event
 *
 * \return		zero on success, negative value if error
 */
iod_ret_t iod_kv_unlink_keys(iod_handle_t oh, iod_trans_id_t tid,
			     iod_hint_list_t *hints, iod_size_t num,
			     iod_kv_params_t *kvs, iod_event_t *event)
{
	IOD_ENTER;


	IOD_EXIT;
	return 0;
}
